"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var JSRuleFinder_1 = require("./shared/JSRuleFinder");
var babelRegisterDone = false;
var registerBabel = function (spin) {
    if (!babelRegisterDone) {
        spin.require('babel-register')({
            presets: [spin.require.resolve('babel-preset-env'), spin.require.resolve('babel-preset-flow')],
            ignore: /node_modules(?!\/(haul|react-native))/,
            retainLines: true,
            sourceMaps: 'inline'
        });
        spin.require('babel-polyfill');
        babelRegisterDone = true;
    }
};
var ReactNativePlugin = (function () {
    function ReactNativePlugin() {
    }
    ReactNativePlugin.prototype.configure = function (builder, spin) {
        var stack = builder.stack;
        if (stack.hasAll(['react-native', 'webpack'])) {
            registerBabel(spin);
            var webpack = spin.require('webpack');
            var mobileAssetTest = /\.(bmp|gif|jpg|jpeg|png|psd|svg|webp|m4v|aac|aiff|caf|m4a|mp3|wav|html|pdf|ttf)$/;
            var AssetResolver = spin.require('haul/src/resolvers/AssetResolver');
            var HasteResolver = spin.require('haul/src/resolvers/HasteResolver');
            var reactNativeRule_1 = {
                loader: spin.require.resolve('babel-loader'),
                options: {
                    babelrc: false,
                    cacheDirectory: spin.dev,
                    compact: !spin.dev,
                    presets: [spin.require.resolve('babel-preset-expo')].concat(spin.dev ? [] : [[spin.require.resolve('babel-preset-minify'), { mangle: false }]]),
                    plugins: [spin.require.resolve('haul/src/utils/fixRequireIssues')]
                }
            };
            var jsRuleFinder_1 = new JSRuleFinder_1.default(builder);
            var jsRule = jsRuleFinder_1.findAndCreateJSRule();
            jsRule.exclude = /node_modules\/(?!react-native.*|@expo|expo|lottie-react-native|haul|pretty-format|react-navigation)$/;
            var origUse_1 = jsRule.use || require.resolve('./shared/identity-loader');
            jsRule.use = function (req) { return (req.resource.indexOf('node_modules') >= 0 ? reactNativeRule_1 : origUse_1); };
            builder.config.resolve.extensions = ["." + stack.platform + ".", '.native.', '.']
                .map(function (prefix) { return jsRuleFinder_1.extensions.map(function (ext) { return prefix + ext; }); })
                .reduce(function (acc, val) { return acc.concat(val); });
            builder.config = spin.merge(builder.config, {
                module: {
                    rules: [
                        { parser: { requireEnsure: false } },
                        {
                            test: mobileAssetTest,
                            use: {
                                loader: require.resolve('./react-native/assetLoader'),
                                query: {
                                    platform: stack.platform,
                                    root: path.resolve('.'),
                                    cwd: spin.cwd,
                                    bundle: false
                                }
                            }
                        }
                    ]
                },
                resolve: {
                    plugins: [
                        new HasteResolver({
                            directories: [spin.require.resolve('react-native')]
                        }),
                        new AssetResolver({
                            platform: stack.platform,
                            test: mobileAssetTest
                        })
                    ],
                    mainFields: ['react-native', 'browser', 'main']
                },
                target: 'webworker'
            });
            var reactVer = spin.require('react-native/package.json').version.split('.')[1] >= 43 ? 16 : 15;
            if (stack.hasAny('dll')) {
                builder.config = spin.merge(builder.config, {
                    entry: {
                        vendor: [spin.require.resolve("spinjs/react-native-polyfills/react-native-polyfill-" + reactVer + ".js")]
                    }
                });
            }
            else {
                var idx = builder.config.entry.index.indexOf('babel-polyfill');
                if (idx >= 0) {
                    builder.config.entry.index.splice(idx, 1);
                }
                builder.config = spin.merge({
                    plugins: [
                        new webpack.SourceMapDevToolPlugin({
                            test: new RegExp("\\.bundle$"),
                            filename: '[file].map'
                        })
                    ],
                    entry: {
                        index: [spin.require.resolve("spinjs/react-native-polyfills/react-native-polyfill-" + reactVer + ".js")]
                    }
                }, builder.config);
            }
        }
    };
    return ReactNativePlugin;
}());
exports.default = ReactNativePlugin;
//# sourceMappingURL=ReactNativePlugin.js.map